/**
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.vew.KexEmailFaxSMS;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;

/**
 * Test des ganzen Modules
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KexEmailFaxSMSTester extends KexEmailFaxSMS {

	/**
	 * Verbindungsdaten
	 */
	public static final String[] CON_DATA = new String[] { "-datenverteiler=localhost:8083", "-benutzer=Tester",
		"-authentifizierung=c:\\passwd", "-debugLevelStdErrText=WARNING", "-debugLevelFileText=WARNING",
		"-EmailFaxSMS=vmExterneMeldungen.test.1", "-maxGroesseWarteschlangen=10", "-warnGroesseWarteschlangen=50" };

	private SystemObject vmem;

	/**
	 * Beispiel Faxnachricht
	 */
	private final String FAX1 = "* TEST FAX 1 *\n" + "----------------------------\n" + "abcdefghijklmnopqrstuvwxyz\n"
			+ "ABCDEFGHIJKLMNOPQRSTUVWXYZ\n" + "°!\"§$%&/()=?`´\n" + "123456789\n" + ",/*-+:<>\n" + "öäüß\n"
			+ "TAB	TAB\n" + "**************************************\n";

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		KexEmailFaxSMS.dav = connection;

		ddPARAMETRIERUNG = new DataDescription(
				KexEmailFaxSMS.dav.getDataModel().getAttributeGroup("atg.versandModulExterneMeldungen"),
				KexEmailFaxSMS.dav.getDataModel().getAspect("asp.parameterSoll"));

		ddVERSANDAUFTRAGFAX = new DataDescription(
				KexEmailFaxSMS.dav.getDataModel().getAttributeGroup("atg.faxVersandAuftrag"),
				KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		ddVERSANDAUFTRAGMAIL = new DataDescription(
				KexEmailFaxSMS.dav.getDataModel().getAttributeGroup("atg.emailVersandAuftrag"),
				KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		ddVERSANDAUFTRAGSMS = new DataDescription(
				KexEmailFaxSMS.dav.getDataModel().getAttributeGroup("atg.smsVersandAuftrag"),
				KexEmailFaxSMS.dav.getDataModel().getAspect("asp.auftrag"));

		if (modulID != 0) {
			vmem = KexEmailFaxSMS.dav.getDataModel().getObject(modulID);
		} else {
			vmem = KexEmailFaxSMS.dav.getDataModel().getObject(modulPID);
		}

		if (vmem == null) {
			throw new Exception("Veersandmodul wurde nicht gefunden");
		}

		VersandStatusSender.getInstanz().init(KexEmailFaxSMS.dav, vmem);

		kompEmail = new KexEmailTexterTest();
		kompFax = new KexFaxTexterTest();
		kompSms = new KexSmsTexterTest(false);

		kompEmail.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
		kompFax.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);
		kompSms.init(KexEmailFaxSMS.dav, vmem, maxGroesseWarteSchlangen);

		kompEmail.start();
		kompFax.start();
		kompSms.start();

		KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddPARAMETRIERUNG, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGFAX, ReceiveOptions.normal(),
				ReceiverRole.drain());
		KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGMAIL, ReceiveOptions.normal(),
				ReceiverRole.drain());
		KexEmailFaxSMS.dav.subscribeReceiver(this, vmem, ddVERSANDAUFTRAGSMS, ReceiveOptions.normal(),
				ReceiverRole.drain());

		// / TEST ZWECKE
		KexEmailFaxSMS.dav.subscribeSender(this, vmem, ddVERSANDAUFTRAGFAX, SenderRole.sender());
		KexEmailFaxSMS.dav.subscribeSender(this, vmem, ddVERSANDAUFTRAGMAIL, SenderRole.sender());
		KexEmailFaxSMS.dav.subscribeSender(this, vmem, ddVERSANDAUFTRAGSMS, SenderRole.sender());

	}

	/**
	 * Sendet einen Fax Auftrag
	 *
	 * @throws Exception
	 */
	public void sendeFaxAuftrag() throws Exception {
		final Data data = KexEmailFaxSMS.dav.createData(ddVERSANDAUFTRAGFAX.getAttributeGroup());

		data.getItem("MeldungsID").getItem("ApplikationsID").asUnscaledValue().set(12345);
		data.getItem("MeldungsID").getItem("LaufendeNummer").asUnscaledValue().set(33333);

		data.getItem("Empfaenger").asTextArray().setLength(3);
		data.getItem("Empfaenger").asTextArray().getTextValue(0).setText("15");
		data.getItem("Empfaenger").asTextArray().getTextValue(1).setText("16");
		data.getItem("Empfaenger").asTextArray().getTextValue(2).setText("17");

		data.getItem("Text").asTextValue().setText(FAX1);

		final ResultData resDatei = new ResultData(vmem, ddVERSANDAUFTRAGFAX, System.currentTimeMillis(), data);
		KexEmailFaxSMS.dav.sendData(resDatei);
	}

	/**
	 * Sendet einen Email Auftrag
	 *
	 * @throws Exception
	 */
	public void sendeMailAuftrag() throws Exception {
		final Data data = KexEmailFaxSMS.dav.createData(ddVERSANDAUFTRAGMAIL.getAttributeGroup());

		data.getItem("MeldungsID").getItem("ApplikationsID").asUnscaledValue().set(1);
		data.getItem("MeldungsID").getItem("LaufendeNummer").asUnscaledValue().set(1);

		data.getItem("Empfaenger").asTextValue().setText("Empf");

		data.getItem("CCEmpfaenger").asTextArray().setLength(3);
		data.getItem("CCEmpfaenger").asTextArray().getTextValue(0).setText("CC_Empf1");
		data.getItem("CCEmpfaenger").asTextArray().getTextValue(1).setText("CC_Empf2");
		data.getItem("CCEmpfaenger").asTextArray().getTextValue(2).setText("CC_Empf3");

		data.getItem("BCCEmpfaenger").asTextArray().setLength(2);
		data.getItem("BCCEmpfaenger").asTextArray().getTextValue(0).setText("BCC_Empf1");
		data.getItem("BCCEmpfaenger").asTextArray().getTextValue(1).setText("BCC_Empf2");

		data.getItem("Absender").asTextValue().setText("SENDER");
		data.getItem("Betreff").asTextValue().setText("BETREFF");

		data.getItem("Text").asTextValue().setText("abcdefgh text text");

		final ResultData resDatei = new ResultData(vmem, ddVERSANDAUFTRAGMAIL, System.currentTimeMillis(), data);
		KexEmailFaxSMS.dav.sendData(resDatei);
	}

	/**
	 * Sendet einen SMS Auftrag
	 *
	 * @throws Exception
	 */
	public void sendeSmsAuftrag() throws Exception {
		final Data data = KexEmailFaxSMS.dav.createData(ddVERSANDAUFTRAGSMS.getAttributeGroup());

		data.getItem("MeldungsID").getItem("ApplikationsID").asUnscaledValue().set(1);
		data.getItem("MeldungsID").getItem("LaufendeNummer").asUnscaledValue().set(1);

		data.getItem("Empfaenger").asTextArray().setLength(2);
		data.getItem("Empfaenger").asTextArray().getTextValue(0).setText("123456");
		data.getItem("Empfaenger").asTextArray().getTextValue(1).setText("666666");

		data.getItem("Text").asTextValue().setText("TEST TEXT");

		final ResultData resDatei = new ResultData(vmem, ddVERSANDAUFTRAGSMS, System.currentTimeMillis(), data);
		KexEmailFaxSMS.dav.sendData(resDatei);
	}

	/**
	 * Die TestMethode
	 */
	@Test
	public void test1() {
		final String connArgs[] = new String[KexEmailFaxSMSTester.CON_DATA.length];
		for (int i = 0; i < KexEmailFaxSMSTester.CON_DATA.length; i++) {
			connArgs[i] = KexEmailFaxSMSTester.CON_DATA[i];
		}
		final KexEmailFaxSMSTester kefs = new KexEmailFaxSMSTester();
		StandardApplicationRunner.run(kefs, connArgs);

		try {
			Thread.sleep(1000);

			kefs.sendeFaxAuftrag();
			kefs.sendeMailAuftrag();
			kefs.sendeSmsAuftrag();

			Thread.sleep(10000);
			kefs.beenden();

		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
}
