/**
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 *
 * Parametriert den TestVersandModul
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KexEmailFaxSmsParametrierung implements StandardApplication, ClientSenderInterface {

	/**
	 * Verbindungsdaten
	 */
	public static final String[] CON_DATA = new String[] { "-datenverteiler=localhost:8083", "-benutzer=Tester",
		"-authentifizierung=c:\\passwd", "-debugLevelStdErrText=WARNING", "-debugLevelFileText=WARNING" };

	/**
	 * Parameter
	 */
	static final String TEST_SMS_EMPFAENGER = "+4915771531080";
	static final String TEST_SMS_PIN = "3311";
	static final String TEST_SMS_COM_PORT = "COM1";
	static final int TEST_SMS_BAUD_RATE = 57600;
	static final int TEST_SMS_TIMEOUT = 10;
	static final String TEST_SMS_INITSEQ = "AT";

	static final String TEST_EML_MAIL_SERVER = "bitctrlserver01";
	static final String TEST_EML_BENUTZER = "bachraty";
	static final String TEST_EML_PASWORT = "bachraty";
	static final String TEST_EML_EMPFAENGER = "<bachraty@bitctrl.de>";

	static final String TEST_FAX_EMPFAENGER = "15";
	static final String TEST_FAX_COM_PORT = "COM1";
	static final int TEST_FAX_BAUD_RATE = 19200;
	static final String TEST_FAX_PIN = "3311";
	static final int TEST_FAX_TIMEOUT = 30;
	static final String TEST_FAX_INITSEQ = "AT&F&C1&D2&K3S7=90M0";

	/**
	 * Datenbeschreibung der Parametrierung
	 */
	private DataDescription DD_PARAM_VMEM;
	private DataDescription DD_PARAM_VMEM_EMAIL;
	private DataDescription DD_PARAM_VMEM_FAX;
	private DataDescription DD_PARAM_VMEM_SMS;

	public static final String ASP_VORGABE = "asp.parameterVorgabe";

	public static final int AUTHENTIFIZIERUNG_KEINE = 0;
	public static final int AUTHENTIFIZIERUNG_LOGIN = 1;

	/**
	 * ModemTyp
	 */
	public static final int MODEM_TYP_FESTNETZ = 0;
	public static final int MODEM_TYP_GSM = 1;
	public static final int MODEM_TYP_GPRS = 2;

	/**
	 * Verbindung zum DAV
	 */
	private ClientDavInterface dav;

	/**
	 * SystemObjekt Versandmodul
	 */
	private SystemObject vmem;

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		// TODO Auto-generated method stub
		dav = connection;
		DD_PARAM_VMEM = new DataDescription(dav.getDataModel().getAttributeGroup("atg.versandModulExterneMeldungen"),
				dav.getDataModel().getAspect(KexEmailFaxSmsParametrierung.ASP_VORGABE));

		DD_PARAM_VMEM_EMAIL = new DataDescription(
				dav.getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenEmail"),
				dav.getDataModel().getAspect(KexEmailFaxSmsParametrierung.ASP_VORGABE));

		DD_PARAM_VMEM_FAX = new DataDescription(
				dav.getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenFax"),
				dav.getDataModel().getAspect(KexEmailFaxSmsParametrierung.ASP_VORGABE));

		DD_PARAM_VMEM_SMS = new DataDescription(
				dav.getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenSms"),
				dav.getDataModel().getAspect(KexEmailFaxSmsParametrierung.ASP_VORGABE));

		vmem = dav.getDataModel().getObject("vmExterneMeldungen.test.1");

		dav.subscribeSender(this, vmem, DD_PARAM_VMEM, SenderRole.sender());
		dav.subscribeSender(this, vmem, DD_PARAM_VMEM_EMAIL, SenderRole.sender());
		dav.subscribeSender(this, vmem, DD_PARAM_VMEM_FAX, SenderRole.sender());
		dav.subscribeSender(this, vmem, DD_PARAM_VMEM_SMS, SenderRole.sender());
	}

	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		// TODO Auto-generated method stub
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * Hauptmethode.
	 *
	 * @param argumente
	 *            Aufrufsparameter
	 */
	public static void main(final String argumente[]) {
		final KexEmailFaxSmsParametrierung efsparam = new KexEmailFaxSmsParametrierung();
		StandardApplicationRunner.run(efsparam, KexEmailFaxSmsParametrierung.CON_DATA);

		try {
			efsparam.parametriereVmem(2, 5);
			efsparam.parametriereEmail(KexEmailFaxSmsParametrierung.TEST_EML_MAIL_SERVER,
					KexEmailFaxSmsParametrierung.AUTHENTIFIZIERUNG_LOGIN,
					KexEmailFaxSmsParametrierung.TEST_EML_BENUTZER, KexEmailFaxSmsParametrierung.TEST_EML_PASWORT,
					"StdSender", "StdEmpfaenger", "StdBetreff");
			efsparam.parametriereFax(KexEmailFaxSmsParametrierung.TEST_FAX_COM_PORT,
					KexEmailFaxSmsParametrierung.TEST_FAX_BAUD_RATE, KexEmailFaxSmsParametrierung.TEST_FAX_TIMEOUT,
					KexEmailFaxSmsParametrierung.MODEM_TYP_FESTNETZ, KexEmailFaxSmsParametrierung.TEST_FAX_PIN,
					KexEmailFaxSmsParametrierung.TEST_FAX_INITSEQ, "", "KEX FAX", 1,
					KexEmailFaxSmsParametrierung.TEST_FAX_EMPFAENGER);
			efsparam.parametriereSMS(KexEmailFaxSmsParametrierung.TEST_SMS_COM_PORT,
					KexEmailFaxSmsParametrierung.TEST_SMS_BAUD_RATE, KexEmailFaxSmsParametrierung.TEST_SMS_TIMEOUT,
					KexEmailFaxSmsParametrierung.MODEM_TYP_GSM, KexEmailFaxSmsParametrierung.TEST_SMS_PIN,
					KexEmailFaxSmsParametrierung.TEST_SMS_INITSEQ, "",
					KexEmailFaxSmsParametrierung.TEST_SMS_EMPFAENGER);
			System.out.println("Versandmodul Fax, Email, SMS parametriert");
			efsparam.disconnect();
		} catch (final Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Parametriert die Verwaltung
	 *
	 * @param nWh
	 *            Max Wiederholungen
	 * @param pause
	 *            Pause zwischen Wiederholungsversuchen
	 * @throws Exception
	 */
	public void parametriereVmem(final int nWh, final int pause) throws Exception {
		Data data;
		ResultData resDatei;
		data = dav.createData(DD_PARAM_VMEM.getAttributeGroup());

		data.getItem("WiederholungsVersuche").asUnscaledValue().set(nWh);
		data.getItem("WiederholungsPause").asUnscaledValue().set(pause);
		resDatei = new ResultData(vmem, DD_PARAM_VMEM, System.currentTimeMillis(), data);

		dav.sendData(resDatei);
	}

	/**
	 * Parametriert den EmailModul
	 *
	 * @param mailServer
	 *            MailServer
	 * @param auth
	 *            Authentifizierung (1 ja, 0 nein)
	 * @param user
	 *            Username
	 * @param pass
	 *            Passwort
	 * @param sender
	 *            Stamdardsender
	 * @param empfaenger
	 *            StandardEmpfaenger
	 * @param betreff
	 *            StandardBetreff
	 * @throws Exception
	 */
	public void parametriereEmail(final String mailServer, final int auth, final String user, final String pass,
			final String sender, final String empfaenger, final String betreff) throws Exception {
		Data data;
		ResultData resDatei;
		data = dav.createData(DD_PARAM_VMEM_EMAIL.getAttributeGroup());

		data.getItem("MailServer").asTextValue().setText(mailServer);
		data.getItem("Authentifizierung").asUnscaledValue().set(auth);
		data.getItem("Benutzer").asTextValue().setText(user);
		data.getItem("Passwort").asTextValue().setText(pass);

		data.getItem("StandardSender").asTextValue().setText(sender);
		data.getItem("StandardEmpfaenger").asTextValue().setText(empfaenger);
		data.getItem("StandardBetreff").asTextValue().setText(betreff);

		resDatei = new ResultData(vmem, DD_PARAM_VMEM_EMAIL, System.currentTimeMillis(), data);
		dav.sendData(resDatei);
	}

	/**
	 * Parametriert den FaxModul
	 *
	 * @param com
	 *            COM-Port
	 * @param baud
	 *            baudrate
	 * @param timeout
	 *            Timeout in s
	 * @param typ
	 *            Modem Typ
	 * @param gsmPin
	 *            PIN-Kode
	 * @param init
	 *            INIT Sequenz
	 * @param prefix
	 *            Praefix
	 * @param faxKennung
	 *            FaxKennung des Versenders
	 * @param hochaufloesend
	 *            True, wenn Fax hochaufloesend versendet werden soll
	 * @param empfaenger
	 *            StandardEmpfaenger
	 * @throws Exception
	 */
	public void parametriereFax(final String com, final int baud, final int timeout, final int typ, final String gsmPin,
			final String init, final String prefix, final String faxKennung, final int hochaufloesend,
			final String empfaenger) throws Exception {
		Data data;
		ResultData resDatei;
		data = dav.createData(DD_PARAM_VMEM_FAX.getAttributeGroup());

		data.getItem("ModemParameter").getItem("COM-Port").asTextValue().setText(com);
		data.getItem("ModemParameter").getItem("BaudRate").asUnscaledValue().set(baud);
		data.getItem("ModemParameter").getItem("EmpfangsTimeout").asUnscaledValue().set(timeout);

		data.getItem("ModemParameter").getItem("ModemTyp").asUnscaledValue().set(typ);

		data.getItem("ModemParameter").getItem("GsmPin").asTextValue().setText(gsmPin);
		data.getItem("ModemParameter").getItem("InitialisierungsBefehl").asTextValue().setText(init);
		data.getItem("ModemParameter").getItem("WahlPraefix").asTextValue().setText(prefix);

		data.getItem("FaxKennung").asTextValue().setText(faxKennung);
		data.getItem("Hochaufloesend").asUnscaledValue().set(hochaufloesend);
		data.getItem("StandardEmpfaenger").asTextValue().setText(empfaenger);

		resDatei = new ResultData(vmem, DD_PARAM_VMEM_FAX, System.currentTimeMillis(), data);
		dav.sendData(resDatei);
	}

	/**
	 * Parametriert den SMS modul
	 *
	 * @param com
	 *            COM-Port
	 * @param baud
	 *            baudrate
	 * @param timeout
	 *            Timeout in s
	 * @param typ
	 *            Modem Typ
	 * @param gsmPin
	 *            PIN-Kode
	 * @param init
	 *            INIT Sequenz
	 * @param prefix
	 *            Praefix
	 * @param empfaenger
	 *            StdEmpfaenger
	 * @throws Exception
	 */
	public void parametriereSMS(final String com, final int baud, final int timeout, final int typ, final String gsmPin,
			final String init, final String prefix, final String empfaenger) throws Exception {
		Data data;
		ResultData resDatei;
		data = dav.createData(DD_PARAM_VMEM_SMS.getAttributeGroup());

		data.getItem("SmsParameter").getItem("ModemParameter").getItem("COM-Port").asTextValue().setText(com);
		data.getItem("SmsParameter").getItem("ModemParameter").getItem("BaudRate").asUnscaledValue().set(baud);
		data.getItem("SmsParameter").getItem("ModemParameter").getItem("EmpfangsTimeout").asUnscaledValue()
		.set(timeout);

		data.getItem("SmsParameter").getItem("ModemParameter").getItem("ModemTyp").asUnscaledValue().set(typ);

		data.getItem("SmsParameter").getItem("ModemParameter").getItem("GsmPin").asTextValue().setText(gsmPin);
		data.getItem("SmsParameter").getItem("ModemParameter").getItem("InitialisierungsBefehl").asTextValue()
		.setText(init);
		data.getItem("SmsParameter").getItem("ModemParameter").getItem("WahlPraefix").asTextValue().setText(prefix);

		data.getItem("StandardEmpfaenger").asTextValue().setText(empfaenger);

		resDatei = new ResultData(vmem, DD_PARAM_VMEM_SMS, System.currentTimeMillis(), data);
		dav.sendData(resDatei);
	}

	/**
	 * Abmelden
	 */
	public void disconnect() {
		dav.disconnect(false, "");
	}
}
