/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007, 2015 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.emailfaxsms.voipfax.texter;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.AbstraktKexVersandKomponente;
import de.bsvrz.kex.emailfaxsms.EmailFaxSmsMsg;
import de.bsvrz.kex.emailfaxsms.IVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.kex.emailfaxsms.VoIPFaxVersandAuftrag;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Bearbeitet VoIP-Fax-Versandauftr&auml;ge, versendet sie mit der Hilfe der
 * Klasse KexVoIPFaxSender.
 *
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: KexVoIPFaxTexter.java 62426 2016-07-01 11:02:48Z peuker $
 */
public class KexVoIPFaxTexter extends AbstraktKexVersandKomponente implements ClientReceiverInterface {

	/**
	 * Datenbeschreibung fuer die Parametrierung.
	 */
	private DataDescription ddParam;
	/**
	 * StandardEmpfaenger.
	 */
	private String stdEmpfaenger;
	/**
	 * SenderKlasse.
	 */
	private KexVoIPFaxSender sender;
	private String stdFaxKennung;
	private boolean stdHochaufloesend;

	private static final String VERSANDMODUL_ATG = "atg.versandModulExterneMeldungenVoIPFax";
	private static final String VOIPPARAMETER = "VoIPParameter";

	/** Attributgruppe Versandauftrag.	*/
	public static final String VERSANDAUFTRAG_ATG = "atg.VoIPFaxVersandAuftrag";

	/**
	 * Konstruktor.
	 */
	public KexVoIPFaxTexter() {
	}

	@Override
	public void init(final ClientDavInterface connection, final SystemObject modul, final int maxWarteschlangeGroesse) {
		super.init(connection, modul, maxWarteschlangeGroesse);

		ddParam = new DataDescription(getDav().getDataModel().getAttributeGroup(VERSANDMODUL_ATG), getDav()
				.getDataModel().getAspect("asp.parameterSoll"));

		getDav().subscribeReceiver(this, modul, ddParam, ReceiveOptions.normal(), ReceiverRole.receiver());
		getLogger().fine("VoIP-Fax-Texter Initialisierung komplett");
	}

	@Override
	protected synchronized VersandStatus sendeAuftrag(final IVersandAuftrag auftrag, final int maxVersuche,
			final long pause) throws InterruptedException {

		// Warten auf die Parametrierung
		if (sender == null) {
			final String txt = MessageFormat.format(EmailFaxSmsMsg.WarningNochNichtParametriert.toString(),
					"VoIP-Fax-Versand");
			LogTools.log(getLogger(), EmailFaxSmsMsg.WarningAllgemein, txt);
			return new VersandStatus(txt, false);
		}

		if (!(auftrag instanceof VoIPFaxVersandAuftrag)) {
			getLogger().warning("VoIPFax-Versand Komponente kann nur VoIPFax-Versandaufträge bearbeiten");
			return VersandStatus.NICHT_VERSENDET;
		}

		final VoIPFaxVersandAuftrag eAuftr = (VoIPFaxVersandAuftrag) auftrag;

		final Collection<String> empfaenger = eAuftr.getEmpfaenger();
		if ((!"".equals(stdEmpfaenger)) && empfaenger.isEmpty()) {
			empfaenger.add(stdEmpfaenger);
		}

		if (empfaenger.isEmpty()) {
			return new VersandStatus("VoIP-Fax-Versandauftrag kein Empfänger angegeben: " + eAuftr, false);
		}

		if ((eAuftr.getText() == null) || "".equals(eAuftr.getText())) {
			return new VersandStatus("VoIP-Fax-Versandauftrag keine Nachricht angegeben: " + eAuftr, false);
		}

		// Erstellen der Nachricht
		final String text = eAuftr.getText();

		int idx = 0;
		final ArrayList<VersandStatus> empfStatus = new ArrayList<VersandStatus>(empfaenger.size());
		for (int i = 0; i < empfaenger.size(); i++) {
			empfStatus.add(VersandStatus.NICHT_VERSENDET);
		}

		VersandStatus status = VersandStatus.NICHT_VERSENDET;

		// fuer jeden Empfaenger maxVersuche
		for (final String empf : empfaenger) {
			for (int i = 0; i < maxVersuche; i++) {
				if ("".equals(empf)) {
					empfStatus.set(idx, VersandStatus.NICHT_VERSENDET);
					break;
				}

				if (i > 0) {
					getLogger().info("VoIP-Fax-Versand nächster Versuch in " + (pause / 1000) + "s");
					Thread.sleep(pause);
				}

				getLogger().info("VoIP-Fax-Auftrag wird ausgeführt für Empfänger '" + empf + "' (" + (i + 1)
						+ ". Versuch von " + maxVersuche + "): " + auftrag);
				status = sender.send(empf, text, false);
				empfStatus.set(idx, status);
				if (status.getVersandErfolg()) {
					getLogger().info("VoIPFax erfolgreich versendet!");
					break;
				}
				getLogger().info("VoIP-Fax-Versand Status: " + status.getVersandFehlerMeldung());
			}
			idx++;
		}

		// Anzahl der Misserfolge
		idx = 0;
		for (final VersandStatus stat : empfStatus) {
			if (!stat.getVersandErfolg()) {
				idx++;
			}
		}

		if (idx > 0) {
			final StringBuffer str = new StringBuffer(512);
			str.append("Fuer folgende Empfaenger konnte der VoIP-Fax-Auftrag nicht versendet werden: ");
			int i = 0;
			for (final String empf : empfaenger) {
				if (empfStatus.get(i).getVersandErfolg()) {
					continue;
				}
				str.append(empf);
				str.append(" [");
				str.append(empfStatus.get(i).getVersandFehlerMeldung());
				str.append("], ");
				i++;
			}
			return new VersandStatus(str.toString(), false);
		}

		return VersandStatus.VERSAND_OK;
	}

	@Override
	public synchronized void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddParam.getAttributeGroup())) {

				LogTools.log(getLogger(), EmailFaxSmsMsg.InfoParameterEmpfangen, "VoIP-Fax");

				final String host = data.getItem(VOIPPARAMETER).getItem("VoIPSenderHost").asTextValue().getText();
				final long port = data.getItem(VOIPPARAMETER).getItem("VoIPSenderPort").asUnscaledValue().longValue();

				stdFaxKennung = data.getItem("FaxKennung").asTextValue().getText();
				final int hochaufloesend = data.getItem("Hochaufloesend").asUnscaledValue().intValue();
				if (hochaufloesend == 1) {
					stdHochaufloesend = true;
				} else {
					stdHochaufloesend = false;
				}

				stdEmpfaenger = data.getItem("StandardEmpfaenger").asTextValue().getText();

				try {
					sender = new KexVoIPFaxSender(host, (int) port);

				} catch (final Exception e) {
					getLogger().warning("Fehler beim Starten von KexVoIPFaxSender: " + e.getMessage());
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexVoiceSender: " + e.getMessage());
				}
			}
		}
	}

}
